from typing import Any, Callable, List, Tuple

from google.cloud import pubsub_v1

from fluidly.pubsub.base_subscriber import setup_base_subscriber
from fluidly.pubsub.message import Message

Subscriptions = List[Tuple[str, Callable[[Message], Any]]]

subscriber = pubsub_v1.SubscriberClient()


def setup_subscriptions(subscriptions: Subscriptions, **kwargs: Any) -> None:
    setup_base_subscriber(subscriber, subscriptions, **kwargs)



from google.cloud import pubsub_v1

from fluidly.pubsub.base_subscriber import Subscriptions, setup_base_subscriber

subscriber = pubsub_v1.SubscriberClient()


def setup_subscriptions(subscriptions: Subscriptions) -> None:
    setup_base_subscriber(subscriber, subscriptions)



import datetime
import json
from collections import namedtuple
from unittest import mock

from fluidly.pubsub.message import Message


def mock_message(payload: str, attributes: dict = None):
    pubsub_message = mock.Mock()
    pubsub_message.data = payload
    pubsub_message.attributes = attributes
    pubsub_message.publish_time = datetime.datetime.now()

    return Message(pubsub_message)


def message_from_dict(payload_dict: dict, attributes: dict = None):
    dict_as_json = json.dumps(payload_dict)
    return mock_message(dict_as_json, attributes)


def message_from_tuple(payload_tuple: namedtuple, attributes: dict = None):
    tuple_as_json = json.dumps(payload_tuple._asdict())
    return mock_message(tuple_as_json, attributes)


def message_from_fixture(path: str, attributes: dict = None):
    with open(path, "r") as fixture:
        return mock_message(fixture.read(), attributes)



import os
from typing import Any, Callable, List, Tuple

from google.cloud.pubsub_v1 import SubscriberClient
from google.cloud.pubsub_v1.subscriber.futures import StreamingPullFuture

from fluidly.pubsub.exceptions import DropMessageException
from fluidly.pubsub.message import Message

GOOGLE_PROJECT = os.getenv("GOOGLE_PROJECT")
APPLICATION_NAME = os.getenv("APPLICATION_NAME")


Deserialiser = Callable[[Message], Any]
MessageHandler = Callable[[Message], Any]
Subscriptions = List[Tuple[str, MessageHandler]]


def setup_base_subscriber(
    subscriber: SubscriberClient, subscriptions: Subscriptions, **kwargs: Any
) -> List[StreamingPullFuture]:
    return [
        subscriber.subscribe(
            subscriber.subscription_path(GOOGLE_PROJECT, subscription_name),
            callback=generate_callback(Message, message_handler),
            **kwargs
        )
        for subscription_name, message_handler in subscriptions
    ]


def generate_callback(
    deserialiser: Deserialiser, message_handler: MessageHandler
) -> Callable[[Message], Any]:
    def callback(message: Message) -> None:
        deserialised_message = deserialiser(message)

        try:
            attributes = dict(message.attributes) if message.attributes else None
            if (
                attributes
                and "audience" in attributes
                and attributes["audience"]
                and APPLICATION_NAME not in attributes["audience"]
            ):
                message.ack()
                return

            message_handler(deserialised_message)

        except DropMessageException:
            message.ack()

    return callback



import json
from typing import Any

from google.cloud.pubsub_v1.types import PubsubMessage

from fluidly.pubsub.exceptions import DropMessageException


class Message:
    def __init__(self, message: PubsubMessage) -> None:
        self.message = message
        self.attributes = dict(message.attributes) if message.attributes else {}
        self.data = json.loads(message.data)
        self.publish_time = message.publish_time

    def __getattr__(self, attr: str) -> Any:
        try:
            return self.data[attr]
        except KeyError as e:
            raise DropMessageException() from e



from typing import List

try:
    import pkg_resources

    pkg_resources.declare_namespace(__name__)
except ImportError:
    import pkgutil

    __path__: List[str] = pkgutil.extend_path(__path__, __name__)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import io
import os

from setuptools import find_packages, setup

NAME = "fluidly-pubsub"
DESCRIPTION = "Pubssub subscription helpers"
URL = "https://github.com/fluidly/python-shared"
EMAIL = "tech@fluidly.com"
AUTHOR = "Fluidly"
REQUIRES_PYTHON = ">=3.6.0"
VERSION = "0.1.0"

REQUIRED = ["google-cloud-pubsub"]

EXTRAS = {
    # 'fancy feature': ['django'],
}

try:
    package_root = os.path.abspath(os.path.dirname(__file__))

    readme_filename = os.path.join(package_root, "README.md")
    with io.open(readme_filename, encoding="utf-8") as readme_file:
        readme = readme_file.read()
except FileNotFoundError:
    readme = DESCRIPTION

packages = [
    package
    for package in find_packages(exclude=["tests", "*.tests", "*.tests.*", "tests.*"])
    if package.startswith("fluidly")
]

setup(
    name=NAME,
    version=VERSION,
    description=DESCRIPTION,
    long_description=readme,
    long_description_content_type="text/markdown",
    author=AUTHOR,
    author_email=EMAIL,
    python_requires=REQUIRES_PYTHON,
    url=URL,
    packages=packages,
    package_data={"fluidly.pubsub": ["py.typed"]},
    namespace_packages=["fluidly"],
    install_requires=REQUIRED,
    extras_require=EXTRAS,
    include_package_data=True,
)



import json
from collections import defaultdict
from typing import Any, Callable, Dict, List, Tuple

from fluidly.pubsub.message import Message
from fluidly.pubsub.tests import message_from_dict


class MessageFuture:
    def result(self) -> None:
        pass


class TopicSpy:
    def __init__(self) -> None:
        self.call_count = 0
        self.call_list: List[str] = []
        self.calls: List[List[Any]] = []

    @property
    def last_published_message_data(self) -> Any:
        return self.calls[-1][0]

    @property
    def last_published_message_attributes(self) -> Any:
        return self.calls[-1][1]


class FakePublisher:
    def __init__(
        self, session: Any, subscriptions: List[Tuple[str, Callable[..., Message]]] = []
    ) -> None:
        self.session = session
        self.subscriptions = dict(subscriptions)
        self.topics_called: Dict[str, TopicSpy] = defaultdict(TopicSpy)

    def publish(
        self, topic: str, data: str, connection_id: str = "", **attrs: Any
    ) -> MessageFuture:

        if topic in self.subscriptions:
            self.subscriptions[topic](self.session, message_from_dict(json.loads(data)))
        self.topics_called[topic].call_count += 1
        self.topics_called[topic].call_list.append(data)
        self.topics_called[topic].calls.append(
            [data, {**attrs, "connection_id": connection_id}]
        )

        return MessageFuture()

    def topic_path(self, project: str, topic: str) -> str:
        return topic



from typing import Optional

from google.cloud.pubsub_v1 import PublisherClient

publisher: Optional[PublisherClient] = None


def get_pubsub_publisher() -> PublisherClient:
    global publisher

    if publisher is None:
        publisher = PublisherClient()

    return publisher
